@@ -16,22 +16,24 @@
 #
 
 
-
+import itertools
 import logging
 import pprint
 import re
 import hashlib
 import base64
-
+import copy
 
 from django.db import models
 from django.db.models import Count, F
+from django.contrib.auth.models import User, Group
 from django.contrib.contenttypes.models import ContentType
 from django.contrib.contenttypes import generic
 from django.core import urlresolvers
 from django.utils.safestring import mark_safe
 from django.utils import timezone
 from django.core.files.base import ContentFile
+from django.core.exceptions import ObjectDoesNotExist
 
 import dingos.read_settings
 
@@ -39,7 +41,7 @@ import dingos
 
 from dingos import *
 
-from dingos.core.datastructures import DingoObjDict,ExtendedSortedDict
+from dingos.core.datastructures import DingoObjDict,ExtendedSortedDict,dict2DingoObjDict
 
 logger = logging.getLogger(__name__)
 pp = pprint.PrettyPrinter(indent=2)
@@ -678,6 +680,7 @@ class InfoObject2Fact(DingoModel):
 
     node_id = models.ForeignKey("NodeID")
 
+    namespace_map = models.ForeignKey("FactTermNamespaceMap",null=True)
 
 
     @property
@@ -703,6 +706,63 @@ class InfoObject2Fact(DingoModel):
 dingos_class_map["InfoObject2Fact"] = InfoObject2Fact
 
 
+class FactTermNamespaceMap(DingoModel):
+    """
+    A mapping of components in a fact term to the namespace from which
+    they originated. For example, consider the following piece of XML::
+
+             <FileObj:FileObjectType>
+           <FileObj:File_Name datatype="String" condition="Equals">UNITED NATIONS COMPENSATION SCHEME...pdf
+           </FileObj:File_Name>
+           <FileObj:Hashes>
+               <Common:Hash>
+                   <Common:Type datatype="String">SHA256</Common:Type>
+                   <Common:Simple_Hash_Value datatype="hexBinary" condition="Equals">
+                   586fea79dd23a352a14c3f8bf3dbc9eb732e1d54f804a29160894aec55df4bd5
+                   </Common:Simple_Hash_Value>
+               </Common:Hash>
+
+    Here, for the fact term `Hashes/Hash/Type`, the 0th component `Hashes` would
+    be mapped to the `FileObj` namespace (or rather, the URI associated with the
+    tag `FileObj`), and the 1st and 2nd components `Hash` and `Type` to the
+    namespace associated with `Common`.
+
+    We keep this information around in case we require it to reproduce XML output.
+    """
+
+    fact_term = models.ForeignKey(FactTerm)
+
+    namespaces = models.ManyToManyField("DataTypeNameSpace",
+                                       through="PositionalNamespace",
+                                       )
+
+    def __unicode__(self):
+        return "%s" % (self.fact_term.term)
+
+dingos_class_map["FactTermNamespaceMap"] = FactTermNamespaceMap
+
+
+class PositionalNamespace(DingoModel):
+    """
+    A through model that allows to annotate a mapping between fact term
+    and namespaces with positional information (thus specifying, with
+    which component in a fact term the namespace should be mapped.
+
+    """
+    fact_term_namespace_map = models.ForeignKey("FactTermNamespaceMap",
+                               related_name="namespaces_thru",
+                             )
+
+    namespace = models.ForeignKey("DataTypeNameSpace",
+                             related_name="fact_term_namespace_map_thru",
+                             )
+    position = models.SmallIntegerField()
+
+    def __unicode__(self):
+        return "%s: %s" % (self.position, self.namespace.uri)
+
+dingos_class_map["PositionalNamespace"] = PositionalNamespace
+
 class Fact(DingoModel):
     """
     In facts, we associate fact terms with one or more fact values.
@@ -847,16 +907,19 @@ class InfoObject(DingoModel):
                             help_text="""Name of the information object, usually auto generated.
                                          from type and facts flagged as 'naming'.""")
 
-    @property
-    def marking_thru(self):
-        """
-        Return the back-pointer to a marking that may
-        has been attached via Django's content type mechanism.
-        """
+    #@property
+    #def marking_thru(self):
+    #    """
+    #    Return the back-pointer to a marking that may
+    #    has been attached via Django's content type mechanism.
+    #    """
 
-        self_django_type = ContentType.objects.get_for_model(self)
-        return Marking2X.objects.filter(content_type__pk=self_django_type.id,
-                                        object_id=self.id)
+    #    self_django_type = ContentType.objects.get_for_model(self)
+    #    return Marking2X.objects.filter(content_type__pk=self_django_type.id,
+    #                                    object_id=self.id)
+
+
+    marking_thru = generic.GenericRelation("Marking2X")
 
 
     class Meta:
@@ -867,6 +930,9 @@ class InfoObject(DingoModel):
     def __unicode__(self):
         return "%s: %s" % (self.iobject_type, self.name)
 
+    def clear(self):
+        self.fact_thru.all().delete()
+
     def is_empty(self):
         """
         Returns true if the enrichment is empty
@@ -913,7 +979,10 @@ class InfoObject(DingoModel):
                  value_iobject_id=None,
                  value_iobject_ts=None,
                  node_id_name='',
-                 is_attribute=False):
+                 is_attribute=False,
+                 ns_uri_dict=None,
+                 namespaces=None,
+                 top_level_namespace=None):
         """
         Add a fact to the iobject. If a fact term for the iobject type
         with the given fact data type and source (CYBOX, etc.) does not
@@ -921,9 +990,16 @@ class InfoObject(DingoModel):
 
         """
 
+
         if not values:
             values = []
 
+        if not namespaces:
+            namespaces = []
+
+        if not ns_uri_dict:
+            ns_uri_dict = None
+
         # get or create fact_term
         fact_term, created = get_or_create_fact_term(iobject_family_name=self.iobject_family.name,
                                                      fact_term_name=fact_term_name,
@@ -937,6 +1013,7 @@ class InfoObject(DingoModel):
                                                      dingos_class_map=self._DCM)
 
 
+
         # get or create fact object
 
         fact_obj, created = get_or_create_fact(fact_term,
@@ -948,6 +1025,7 @@ class InfoObject(DingoModel):
                                                )
 
 
+
         # get or create node identifier
 
         node_id, created = dingos_class_map['NodeID'].objects.get_or_create(name=node_id_name)
@@ -967,19 +1045,75 @@ class InfoObject(DingoModel):
             except InfoObject2Fact.DoesNotExist:
                 pass
 
-        e2f = self._DCM['InfoObject2Fact'].objects.create(
+        io2f = self._DCM['InfoObject2Fact'].objects.create(
             node_id=node_id,
             iobject=self,
             fact=fact_obj,
             attributed_fact=attributed_io2f)
 
-        return e2f
+        counter = 0
+        io2f2n_list = []
+
+
+        namespace_map_elts = FactTermNamespaceMap.objects.filter(fact_term=fact_term).order_by('id','namespaces_thru__position').values_list('id',
+                                                                                             'namespaces_thru__position',
+                                                                                             'namespaces_thru__namespace__uri')
+
+        namespace_maps = []
+        for k,g in itertools.groupby(namespace_map_elts,lambda x:x[0]):
+            namespace_maps.append(list(g))
+
+
+        namespace_map = None
+
+        namespaces_uris = map(lambda x:x[0],namespaces)
+
+
+        for i in namespace_maps:
+            if namespaces_uris == map(lambda x:x[2],i):
+                namespace_map = FactTermNamespaceMap.objects.get(id=i[0][0])
+
+
+                break
+
+
+
+        if not namespace_map:
+
+            namespace_map = FactTermNamespaceMap.objects.create(fact_term=fact_term)
+            for (ns_uri,ns_slug) in namespaces:
+
+                if ns_uri:
+
+                    if not (ns_uri in ns_uri_dict):
+                        ns_uri_obj, created = self._DCM['DataTypeNameSpace'].objects.get_or_create(uri=ns_uri,defaults={'name':ns_slug})
+                        ns_uri_obj_pk = ns_uri_obj.pk
+                        ns_uri_dict[ns_uri]=ns_uri_obj_pk
+                    else:
+                        ns_uri_obj_pk = ns_uri_dict[ns_uri]
+
+                    io2f2n_list.append(PositionalNamespace(fact_term_namespace_map=namespace_map,position=counter,namespace_id=ns_uri_obj_pk))
+
+                counter +=1
+                #self._DCM['IO2F2Namespace'].object.create(io2f=self,position=counter,namespace__pk=ns_uri_obj_pk)
+
+            if len(io2f2n_list)!= 0:
+                PositionalNamespace.objects.bulk_create(io2f2n_list)
+            else:
+                namespace_map.delete()
+                namespace_map = None
+
+        if namespace_map:
+            io2f.namespace_map = namespace_map
+            io2f.save()
+
+        return io2f
 
     def from_dict(self,
                   dingos_obj_dict,
                   config_hooks=None,
                   namespace_dict=None,
-                  special_ft_handler=None):
+                  ):
         """
         Convert DingoObjDict to facts and associate resulting facts with this information object.
         """
@@ -987,6 +1121,14 @@ class InfoObject(DingoModel):
 
         # Instantiate default parameters
 
+
+
+
+        if '@@ns' in dingos_obj_dict.keys():
+            top_level_namespace = (namespace_dict.get(dingos_obj_dict.get('@@ns'),None),dingos_obj_dict.get('@@ns'))
+        else:
+            top_level_namespace = (None,None)
+
         if not config_hooks:
             config_hooks = {}
 
@@ -1001,14 +1143,24 @@ class InfoObject(DingoModel):
         if not namespace_dict:
             namespace_dict = {}
 
+
+        namespace_uri_2_pk_mapping = dict(self._DCM['DataTypeNameSpace'].objects.values_list('uri','id'))
+
+
         if not self.is_empty():
-            logger.error("Attempt to import a dictionary into a non-empty info object")
-            return
+            logger.debug("Non-empty info object %s (timestamp %s, pk %s ) is overwritten with new information" % (self.identifier,
+                                                                                                                 self.timestamp,
+                                                                                                                 self.pk))
+            self.clear()
+
 
         # Flatten the DingoObjDict
 
         (flat_list, attrs) = dingos_obj_dict.flatten(attr_ignore_predicate=attr_ignore_predicate,
-                                                     force_nonleaf_fact_predicate=force_nonleaf_fact_predicate)
+                                                     force_nonleaf_fact_predicate=force_nonleaf_fact_predicate,
+                                                     namespace_dict=namespace_dict)
+
+
 
         for fact in flat_list:
 
@@ -1072,6 +1224,8 @@ class InfoObject(DingoModel):
             add_fact_kargs['fact_term_attribute'] = fact['attribute']
             add_fact_kargs['values'] = [fact['value']]
             add_fact_kargs['node_id_name'] = fact['node_id']
+            add_fact_kargs['namespaces'] = fact['namespaces']
+            add_fact_kargs['top_level_namespace'] = top_level_namespace
 
             handler_return_value = True
 
@@ -1090,6 +1244,7 @@ class InfoObject(DingoModel):
                             break
             logger.debug("Treating fact (before special handler list) %s with attr_info %s and kargs %s" % (fact, attr_info, add_fact_kargs))
             if (handler_return_value == True):
+                add_fact_kargs['ns_uri_dict'] = namespace_uri_2_pk_mapping
                 e2f_obj = self.add_fact(**add_fact_kargs)
             elif not handler_return_value:
                 continue
@@ -1098,56 +1253,138 @@ class InfoObject(DingoModel):
         self.set_name()
 
 
-    def to_dict(self,include_node_id=False):
+
+
+    def to_dict(self,include_node_id=False,no_attributes=False,track_namespaces=False):
+        """
+        This function is currently geared very much towards writing
+        STIX/CybOX objects to a dictionary. That should not be the case -- the function
+        needs to be generic just as the from_dict function.
+        TODO: make function generic.
+        """
         flat_result = []
 
-        fact_thrus = self.fact_thru.all().prefetch_related(
-                                                           'fact__fact_term',
-                                                           'fact__fact_values',
-                                                           'fact__fact_values__fact_data_type',
-                                                           'fact__fact_values__fact_data_type__namespace',
-                                                           'fact__value_iobject_id',
-                                                           'fact__value_iobject_id__namespace',
-                                                           'node_id')
+        def make_ns_slug(name_counter,slug='n'):
+            while "%s%s" % (slug,name_counter['counter']) in namespace_mapping.values():
+                name_counter['counter']  = name_counter['counter']+1
 
+            return "%s%s" % (slug,name_counter['counter'])
+
+        name_counter = {'counter':0}
+
+        if track_namespaces:
+
+            fact_thrus = self.fact_thru.all().prefetch_related(
+                'fact__fact_term',
+                'fact__fact_values',
+                'fact__fact_values__fact_data_type',
+                'fact__fact_values__fact_data_type__namespace',
+                'fact__value_iobject_id',
+                'fact__value_iobject_id__namespace',
+                'namespace_map__namespaces_thru__namespace',
+                'node_id')
+        else:
+            fact_thrus = self.fact_thru.all().prefetch_related(
+                'fact__fact_term',
+                'fact__fact_values',
+                'fact__fact_values__fact_data_type',
+                'fact__fact_values__fact_data_type__namespace',
+                'fact__value_iobject_id',
+                'fact__value_iobject_id__namespace',
+                'node_id')
+
+        export_ns_dict = {}
+
+        namespace_mapping=  {"%s-%s" % (self.iobject_type.namespace.uri,self.iobject_type_revision): 'n0'}
         #fact_thrus = self.fact_thru.all()
         for fact_thru in fact_thrus:
+            #print fact_thru.node_id
+            #print fact_thru.fact.fact_term
+            #for positional_namespace in fact_thru.namespace_map.namespaces_thru.all():
+            #    print positional_namespace
+
             value_list = []
             first = True
             fact_datatype_name = None
             fact_datatype_ns = None
-            for fact_value in fact_thru.fact.fact_values.all():
-                if first:
-                    fact_datatype_name = fact_value.fact_data_type.name
-                    fact_datatype_ns = fact_value.fact_data_type.namespace.uri
-                    if (fact_datatype_name == DINGOS_DEFAULT_FACT_DATATYPE and
-                        fact_datatype_ns == DINGOS_NAMESPACE_URI):
-
-                        fact_datatype_name = None
-                        fact_datatype_ns = None
-                        first = False
-                value_list.append(fact_value.value)
 
             fact_dict = {'node_id': fact_thru.node_id.name,
                          'term': fact_thru.fact.fact_term.term,
                          'attribute' : fact_thru.fact.fact_term.attribute,
-                         '@@type': fact_datatype_name,
-                         '@@type_ns': fact_datatype_ns,
-                         'value_list': value_list}
+                         '@@namespace_map' : fact_thru.namespace_map,
+                         }
+
+            for fact_value in fact_thru.fact.fact_values.all():
+                if first:
+                    first=False
+                    fact_datatype_name = fact_value.fact_data_type.name
+                    fact_datatype_ns = fact_value.fact_data_type.namespace.uri
+                    if (fact_datatype_name == DINGOS_DEFAULT_FACT_DATATYPE and
+                                fact_datatype_ns == DINGOS_NAMESPACE_URI) or fact_thru.fact.value_iobject_id:
+                        pass
+                    else:
+                        if not fact_datatype_ns in namespace_mapping:
+                            namespace_slug = make_ns_slug(name_counter)
+                            namespace_mapping[fact_datatype_ns] = namespace_slug
+                        else:
+                            namespace_slug= namespace_mapping[fact_datatype_ns]
+                        fact_dict['@@type'] = '%s:%s' % (namespace_slug,fact_datatype_name)
+
+                value_list.append(fact_value.value)
+
+
+
+
+            fact_dict['value_list'] = value_list
+
 
             if fact_thru.fact.value_iobject_id:
                 value_iobject_id_ns = fact_thru.fact.value_iobject_id.namespace.uri
+                if not value_iobject_id_ns in namespace_mapping:
+                    namespace_slug = make_ns_slug(name_counter)
+                    namespace_mapping[value_iobject_id_ns] = namespace_slug
+                else:
+                    namespace_slug= namespace_mapping[value_iobject_id_ns]
+
                 value_iobject_id  =fact_thru.fact.value_iobject_id.uid
-                fact_dict['@@idref_ns'] = fact_thru.fact.value_iobject_id.namespace.uri
-                fact_dict['@@idref_id'] = fact_thru.fact.value_iobject_id.uid
+                if fact_dict['attribute']:
+                    # Here we treat the case that the reference is part of an attribute such as
+                    # 'phase_id'
+                    fact_dict['value_list'] = ["%s:%s" % (namespace_slug,value_iobject_id)]
+                else:
+                    # Otherwise, we sneak in an idref attribute. Because the code that
+                    # generates the dictionary simply dumps all untreated key-value paris
+                    # into the generated dictionary, this works... but is a bit of a hack, really.
+
+                    fact_dict['@idref'] = "%s:%s" % (namespace_slug,value_iobject_id)
+
             flat_result.append(fact_dict)
 
         result = DingoObjDict()
-        result.from_flat_repr(flat_result,include_node_id=include_node_id)
-        result['@@iobject_type'] = self.iobject_type.name
-        result['@@iobject_type_ns'] = self.iobject_type.namespace.uri
-        #result = result.to_tuple()
-        return result
+
+
+        result.from_flat_repr(flat_result,
+            include_node_id=include_node_id,
+            no_attributes=no_attributes,
+            track_namespaces=track_namespaces,
+            namespace_mapping=namespace_mapping
+            )
+
+
+        if not no_attributes:
+            if not track_namespaces:
+                result['@@iobject_type'] = self.iobject_type.name
+                result['@@iobject_type_ns'] = self.iobject_type.namespace.uri
+                return result
+            else:
+                result['@ns'] = namespace_mapping["%s-%s" % (self.iobject_type.namespace.uri,self.iobject_type_revision)]
+                #result['@@iobject_type'] = self.iobject_type.name
+                return {'namespaces': dict(map(lambda x : (x[1],x[0]), namespace_mapping.items())),
+                        'objects' : [result]
+                }
+        else:
+            return result
+
 
     def show_fact_terms(self,level):
         """Returns a list of the fact terms (split in 'term' and 'attribute') that
@@ -1186,17 +1423,23 @@ class InfoObject(DingoModel):
                                                         'fact_term__term',
                                                         'fact_term__attribute',
                                                         'fact_values__value',
+                                                        'fact_values__storage_location',
                                                         'value_iobject_id__latest__name')
 
         # We build a dictionary that will then be used for the format string
 
         fact_dict = {}
         counter = 0
-        for (node_id, fact_term, attribute, value, related_obj_name) in fact_list:
+        for (node_id, fact_term, attribute, value, storage_location, related_obj_name) in fact_list:
             #print fact_list[counter]
             if type(value)==type([]):
                 value = "%s,..." % value
-            if related_obj_name:
+            if storage_location == dingos.DINGOS_BLOB_TABLE:
+                try:
+                    value = BlobStorage.objects.get(sha256=value).content
+                except:
+                    value = "In blob table %s" % value
+            elif related_obj_name:
                 value = related_obj_name
 
             if attribute:
@@ -1412,16 +1655,18 @@ class Relation(DingoModel):
                                     related_name='+',
                                     help_text="InfoObject containing metadata about relation.")
 
-    @property
-    def marking_thru(self):
-        """
-        Return the back-pointer to markings that may have
-        been attached via Django's content type mechanism.
-        """
+    #@property
+    #def prog_marking_thru(self):
+    #    """
+    #    Return the back-pointer to markings that may have
+    #    been attached via Django's content type mechanism.
+    #    """
 
-        self_django_type = ContentType.objects.get_for_model(self)
-        return Marking2X.objects.filter(content_type__pk=self_django_type.id,
-                                        object_id=self.id)
+    #   self_django_type = ContentType.objects.get_for_model(self)
+    #    return Marking2X.objects.filter(content_type__pk=self_django_type.id,
+    #                                    object_id=self.id)
+
+    marking_thru = generic.GenericRelation("Marking2X")
 
 
     class Meta:
@@ -1464,6 +1709,153 @@ class Marking2X(models.Model):
 
 dingos_class_map["Marking2X"] = Marking2X
 
+class UserData(DingoModel):
+    """
+    Model for binding user/group-specific data stored in DINGOS InfoObjects to a Django user/group
+    (or combination thereof).
+    """
+
+    user = models.ForeignKey(User,
+                             null=True)
+    group = models.ForeignKey(Group,
+                              null=True)
+
+    data_kind = models.SlugField(max_length=32)
+
+    identifier = models.ForeignKey(Identifier,
+                                   null=True)
+
+    class Meta:
+        unique_together = ('user', 'group','data_kind')
+
+    def retrieve(self):
+
+        settings_iobject = None
+        if self.identifier:
+            settings_iobject = self.identifier.latest
+
+        if settings_iobject:
+            settings= settings_iobject.to_dict(no_attributes=True,track_namespaces=False)
+            return settings
+        else:
+            return None
+
+    def store(self,settings,iobject_type_name=DINGOS_USER_DATA_TYPE_NAME,keep_history=False,iobject_name=None):
+
+        settings_dod = dict2DingoObjDict(settings)
+
+
+        settings_iobject = None
+        if self.identifier:
+            settings_iobject = self.identifier.latest
+
+        if (not settings_iobject) or keep_history:
+            # We make a new object
+            if self.identifier:
+                iobject_identifier = self.identifier.uid
+            else:
+                # Actually, the identifier of the information object in which
+                # we store user data is irrelevant, since we will always retrieve
+                # the information object / identifier via the foreign key. But
+                # why not make a speaking identifier?
+
+                if self.user:
+                    user_pk = self.user.pk
+                else:
+                    user_pk = None
+
+                if self.group:
+                    group_pk = self.group.pk
+                else:
+                    group_pk = None
+
+                iobject_identifier = "%s_u%s_g%s" % (iobject_type_name,user_pk,group_pk)
+
+
+                settings_iobject,created = get_or_create_iobject(iobject_identifier,
+                                                             identifier_namespace_uri = DINGOS_ID_NAMESPACE_URI,
+                                                             iobject_type_name = iobject_type_name,
+                                                             iobject_type_namespace_uri = DINGOS_NAMESPACE_URI,
+                                                             iobject_type_revision_name = REVISION,
+                                                             iobject_family_name = DINGOS_INTERNAL_IOBJECT_FAMILY_NAME,
+                                                             iobject_family_revision_name= REVISION,
+                                                             identifier_namespace_name= DINGOS_ID_NAMESPACE_SLUG,
+                                                             timestamp=timezone.now(),
+                                                             create_timestamp=timezone.now(),
+                                                             )
+
+        else:
+            # We are going to overwrite information in an existing information object --
+            # let us therefore adjust the timestamp.
+            settings_iobject.timestamp = timezone.now()
+            settings_iobject.save()
+        if not self.identifier:
+            self.identifier = settings_iobject.identifier
+            self.save()
+        result = settings_iobject.from_dict(settings_dod)
+
+        if iobject_name:
+            settings_iobject.name = iobject_name
+            settings_iobject.save()
+        return result
+
+    @staticmethod
+    def get_user_data(user=None,group=None,data_kind=DINGOS_USER_DATA_TYPE_NAME):
+        """
+        Returns either stored settings of a given user or default settings.
+        This behavior reflects the need for views to have some settings at
+        hand when running. The settings are returned as dict object.
+        """
+        logger.debug("Get user settings called")
+
+        if not user.is_authenticated():
+            user = None
+
+        try:
+            user_config =  UserData.objects.get(user=user,group=group,data_kind=data_kind)
+            return user_config.retrieve()
+        except:
+            return None
+
+
+    @staticmethod
+    def get_user_data_iobject(user=None,group=None,data_kind=DINGOS_USER_DATA_TYPE_NAME):
+        """
+        Returns either stored settings of a given user or default settings.
+        This behavior reflects the need for views to have some settings at
+        hand when running. The settings are returned as dict object.
+        """
+        logger.debug("Get user settings called")
+
+        if not user.is_authenticated():
+            user = None
+        try:
+            user_config =  UserData.objects.get(user=user,group=group,data_kind=data_kind)
+            return user_config.identifier.latest
+        except:
+            return None
+
+
+    @staticmethod
+    def store_user_data(user=None, group=None,data_kind=DINGOS_USER_DATA_TYPE_NAME,user_data=None,iobject_name=None):
+        """
+        Returns either stored settings of a given user or default settings.
+        This behavior reflects the need for views to have some settings at
+        hand when running. The settings are returned as dict object.
+        """
+
+        if not user_data:
+            user_data = {}
+
+        if not user.is_authenticated():
+            user = None
+
+        user_config,created =  UserData.objects.get_or_create(user=user,group=group,data_kind=data_kind)
+        return user_config.store(user_data,iobject_type_name=data_kind,iobject_name=iobject_name)
+
+dingos_class_map["UserData"] = UserData
+
+
 
 def get_or_create_iobject(identifier_uid,
                           identifier_namespace_uri,
@@ -1485,6 +1877,9 @@ def get_or_create_iobject(identifier_uid,
 
     # create or retrieve identifier
 
+    if not timestamp:
+        raise StandardError("You must supply a timestamp.")
+
     id_namespace, created = dingos_class_map['IdentifierNameSpace'].objects.get_or_create(uri=identifier_namespace_uri)
 
     if created and identifier_namespace_name:
@@ -1509,25 +1904,26 @@ def get_or_create_iobject(identifier_uid,
 
     if not create_timestamp:
         create_timestamp = timezone.now()
-    if not timestamp:
-        timestamp = create_timestamp
-
-    if overwrite:
-        iobject = overwrite
-        created = False
+    #if not timestamp:
+    #    timestamp = create_timestamp
+    #    iobject = overwrite
+    #    created = False
 
 
-    else:
-        iobject, created = dingos_class_map["InfoObject"].objects.get_or_create(identifier=identifier,
-                                                                               timestamp=timestamp,
-                                                                               defaults={'iobject_family': iobject_family,
-                                                                                         'iobject_family_revision': iobject_family_revision,
-                                                                                         'iobject_type': iobject_type,
-                                                                                         'iobject_type_revision': iobject_type_revision,
-                                                                                         'create_timestamp': create_timestamp})
+
+    iobject, created = dingos_class_map["InfoObject"].objects.get_or_create(identifier=identifier,
+                                                                           timestamp=timestamp,
+                                                                           defaults={'iobject_family': iobject_family,
+                                                                                     'iobject_family_revision': iobject_family_revision,
+                                                                                     'iobject_type': iobject_type,
+                                                                                     'iobject_type_revision': iobject_type_revision,
+                                                                                     'create_timestamp': create_timestamp})
     if created:
         iobject.set_name()
         iobject.save()
+        identifier.latest = iobject
+        identifier.save()
+
 
     elif overwrite:
         iobject.timestamp = timestamp
@@ -1540,7 +1936,7 @@ def get_or_create_iobject(identifier_uid,
         iobject.save()
 
     logger.debug(
-        "Created iobject with %s (created was %s) and %s (overwrite %s)" % (iobject.identifier, timestamp, created, overwrite))
+        "Created iobject id with %s , ts %s (created was %s) and overwrite as %s" % (iobject.identifier, timestamp, created, overwrite))
     return iobject, created
 
 
@@ -1711,3 +2107,6 @@ def write_large_value(value,storage_location=dingos.DINGOS_LARGE_VALUE_DESTINATI
                                                               content=value)
     return (value_hash,storage_location)
 
+
+
+
