@@ -49,7 +49,7 @@ else:
 
 if settings.configured and 'DINGOS' in dir(settings):
     dingos.DINGOS_MAX_VALUE_SIZE_WRITTEN_TO_VALUE_TABLE = settings.DINGOS.get('DINGOS_MAX_VALUE_SIZE_WRITTEN_TO_VALUE_TABLE',
-                                                                             dingos.DINGOS_MAX_VALUE_SIZE_WRITTEN_TO_VALUE_TABLE)
+                                                                              dingos.DINGOS_MAX_VALUE_SIZE_WRITTEN_TO_VALUE_TABLE)
 
 if settings.configured and 'DINGOS' in dir(settings):
     configured_large_value_dest = settings.DINGOS.get('LARGE_VALUE_DESTINATION',None)
@@ -61,4 +61,11 @@ if settings.configured and 'DINGOS' in dir(settings):
     elif configured_large_value_dest == 'DINGOS_BLOB_TABLE':
         dingos.DINGOS_LARGE_VALUE_DESTINATION = dingos.DINGOS_BLOB_TABLE
 
+if settings.configured and 'DINGOS' in dir(settings):
+    dingos.DINGOS_DEFAULT_USER_PREFS = settings.DINGOS.get('DINGOS_DEFAULT_USER_PREFS',
+                                                           dingos.DINGOS_DEFAULT_USER_PREFS)
+
+if settings.configured and 'DINGOS' in dir(settings):
+    dingos.DINGOS_DEFAULT_SAVED_SEARCHES = settings.DINGOS.get('DINGOS_DEFAULT_SAVED_SEARCHES',
+                                                           dingos.DINGOS_DEFAULT_SAVED_SEARCHES)
 
